/*
(C) Umang Gajera | Power_user_EX - www.ocfreaks.com 2011-13.
More Embedded tutorials @ www.ocfreaks.com/cat/embedded

LPC2148 Interrupt Example for Case #3 ,
@ http://www.ocfreaks.com/lpc2148-interrupt-tutorial/

License : GPL.
*/

#include <lpc214x.h>

#define PLOCK 0x00000400
#define MR0I (1<<0) //Interrupt When TC matches MR0
#define MR0R (1<<1)	//Reset TC when TC matches MR0
#define MR0_DELAY_MS 500 //0.5 Second(s) Delay

#define PRESCALE 60000 //60000 PCLK clock cycles to increment TC by 1 

void initClocks(void);
void initTimer0(void);
void initUart0(void);
__irq void myTimer0_ISR(void);
__irq void myUart0_ISR(void);

void setupPLL0(void);
void feedSeq(void);
void connectPLL0(void);


int main(void)
{
	initClocks(); //Initialize CPU and Peripheral Clocks @ 60Mhz
	initTimer0(); //Initialize Timer0
	initUart0(); //Initialize Uart0
	IO0DIR = 0xFFFFFFFF; //Configure all pins as outputs , though we'll use only P0.2,P0.3
	IO0PIN = 0x0;

	while(1); //Infinite Idle Loop

	//return 0; //normally this wont execute ever	:P
}

void initTimer0(void)
{
	/*Assuming that PLL0 has been setup with CCLK = 60Mhz and PCLK also = 60Mhz.*/
   
	//----------Configure Timer0-------------

	T0CTCR = 0x0;	
	
	T0PR = PRESCALE-1; //60000 clock cycles @60Mhz = 1 mS
	
	T0MR0 = MR0_DELAY_MS-1; // 0.5sec (Value in Decimal!) Zero Indexed Count - hence subtracting 1

	T0MCR = MR0I | MR0R; //Set the Match control register 

	//----------Setup TIMER0 Interrupt-------------

	//Using Slot 0 for TIMER0

	VICVectAddr0 = (unsigned)myTimer0_ISR; //Pointer Interrupt Function (ISR)

	VICVectCntl0 = 0x20 | 4; 

	VICIntEnable |= (1<<4); //Enable timer0 int , 4th bit=1
	
	T0TCR = 0x02; //Reset Timer
	
	T0TCR = 0x01; //Enable timer
}

void initUart0(void)
{
	PINSEL0 = 0x5; // Select Txd and Rxd , Txd is not used in our case.

  //If you'r not acquianted to UARTs then use below settings as it is ,
	//I'll cover UARTs and its related Interrupt programming in upcoming Tutorial.
	//Below Settings Assume PCLK and CCLK are running at 60Mhz!
	
	U0LCR = 0x83; // set DLAB=1 so we can set divisor bits
	U0DLL = 0xC3;
	U0DLM = 0x00; 
	U0FDR = 0x10;
	//The above setting will set Baud rate to 19.2k - its just my Preference
	
	U0FCR = 0x6; // reset Rx and Tx FIFO
	U0FCR = 0x1; // enable FIFO
	U0LCR = 0x03; // now make DLAB=0 
	
	U0IER = 0x1; // to keep things simple just enable "Receive Data Available(RDA)" interrupt

	//----------Setup UART0 Interrupt-------------

	//Any Slot with Lower Priority than TIMER0's slot will suffice

	VICVectAddr1 = (unsigned)myUart0_ISR; //Pointer Interrupt Function (ISR)

	VICVectCntl1 = 0x20 | 6; 

	VICIntEnable |= (1<<6); //Enable Uart0 interrupt , 6th bit=1 	
}

__irq void myTimer0_ISR(void)
{
	long int regVal;
	regVal = T0IR; // read the current value in T0's Interrupt Register
	
	IO0PIN ^= (1<<2); // Toggle 3rd Pin in PORT2 .. P0.2

	T0IR = regVal; // write back to clear the interrupt flag
	VICVectAddr = 0x0; // Acknowledge that ISR has finished execution
}

__irq void myUart0_ISR(void)
{
	long int regVal;
	regVal = U0IIR; // Reading U0IIR also clears it!
	
	//Recieve Data Available Interrupt has occured
	regVal = U0RBR; // dummy read
		
	IO0PIN ^= (1<<3); // Toggle 4th Pin in PORT0 .. P0.3

	VICVectAddr = 0x0; // Acknowledge that ISR has finished execution
}	


void initClocks(void)
{
	setupPLL0();
	feedSeq(); //sequence for locking PLL to desired freq.
	connectPLL0();
	feedSeq(); //sequence for connecting the PLL as system clock
   
	//SysClock is now ticking @ 60Mhz!
       
	VPBDIV = 0x01; // PCLK is same as CCLK i.e 60Mhz

	//PLL0 Now configured!
}

//---------PLL Related Functions :---------------

//Using PLL settings as shown in : http://www.ocfreaks.com/lpc214x-pll-tutorial-for-cpu-and-peripheral-clock/

void setupPLL0(void)
{
	//Note : Assuming 12Mhz Xtal is connected to LPC2148.
	
	PLL0CON = 0x01; 
	PLL0CFG = 0x24; 
}

void feedSeq(void)
{
	PLL0FEED = 0xAA;
	PLL0FEED = 0x55;
}

void connectPLL0(void)
{
	while( !( PLL0STAT & PLOCK ));
	PLL0CON = 0x03;
}
